/*******************************************************************************
 * Copyright 2019 Graz University of Technology
 * MOA ZS has been developed in a cooperation between EGIZ
 * and Graz University of Technology.
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 *******************************************************************************/
package at.gv.egiz.moazs.backend;

import at.gv.egiz.moazs.scheme.MsgResponse;
import at.gv.zustellung.app2mzs.xsd.MsgResponseSinksType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.TaskExecutor;
import org.springframework.stereotype.Component;

import static java.util.concurrent.CompletableFuture.supplyAsync;

/**
 * @author Christof Rabensteiner
 *
 */
@Component
public class MsgResponseSinkHub {

    private static final Logger log = LoggerFactory.getLogger(MsgResponseSinkHub.class);

    private final SaveResponseToFileSink saveResponseSink;
    private final LogResponseSink logResponseSink;
    private final ForwardResponseToServiceSink forwardResponseSink;
    private final TaskExecutor taskExecutor;

    @Autowired
    public MsgResponseSinkHub(SaveResponseToFileSink saveResponseSink, LogResponseSink logResponseSink,
                              ForwardResponseToServiceSink forwardResponseSink, TaskExecutor taskExecutor) {
        this.saveResponseSink = saveResponseSink;
        this.logResponseSink = logResponseSink;
        this.forwardResponseSink = forwardResponseSink;
        this.taskExecutor = taskExecutor;
    }

    /**
     * Apply all sinks that are configured in {@code sinkParams} to {@code msgResponse}.
     */
    public void applySinks(MsgResponse msgResponse, MsgResponseSinksType sinkParams) {

        log.info("Apply Sinks to msg:{} with appDeliveryID={}.",
                msgResponse.getRootElementLocalPart(), msgResponse.getAppDeliveryID());

        if (sinkParams.getSaveResponseToFile().isActive()) {
            supplyAsync(() -> saveResponseSink.save(msgResponse, sinkParams.getSaveResponseToFile().getPath()), taskExecutor);
        }

        if (sinkParams.isLogResponse()) {
            supplyAsync(() -> logResponseSink.log(msgResponse), taskExecutor);
        }

        if (sinkParams.getForwardResponseToService().isActive()) {
            supplyAsync(() -> forwardResponseSink.send(
                    msgResponse, sinkParams.getForwardResponseToService().getAppClient()), taskExecutor);
        }

    }
}
